@@ -1,5 +1,5 @@
 package Net::Whois::Raw;
-$Net::Whois::Raw::VERSION = '2.93';
+$Net::Whois::Raw::VERSION = '2.99037';
 # ABSTRACT: Get Whois information of domains and IP addresses.
 
 require 5.008_001;
@@ -73,7 +73,9 @@ sub whois {
         } elsif ($which_whois eq 'QRY_LAST' || !defined($which_whois)) {
             $res_text = $res->[-1]->{text};
             $res_srv  = $res->[-1]->{srv};
-        }
+        } elsif ($which_whois eq 'QRY_ALL') {
+            return $res;
+        }  
     }
     else {
         ($res_text, $res_srv) = get_whois($dom, $server, $which_whois);
@@ -166,36 +168,37 @@ sub _referral_server {
 }
 
 sub recursive_whois {
-    my ($dom, $srv, $was_srv, $norecurse, $is_ns) = @_;
+    my ( $dom, $srv, $was_srv, $norecurse, $is_ns ) = @_;
 
     my $lines = whois_query( $dom, $srv, $is_ns );
-    my $whois = join("", @{$lines});
+    my $whois = join '', @$lines;
 
-    my ($newsrv, $registrar);
-    foreach (@{$lines}) {
-            $registrar ||= /Registrar/ || /Registered through/;
+    my ( $newsrv, $registrar );
+    for ( @$lines ) {
+        $registrar ||= /Registrar/ || /Registered through/;
 
-        if ( $registrar && !$norecurse && /Whois Server:\s*([A-Za-z0-9\-_\.]+)/ ) {
+        # Skip urls as recursive whois servers
+        if ( $registrar && !$norecurse && /whois server:\s*([a-z0-9\-_\.]+)\b/i ) {
             $newsrv = lc $1;
         }
-        elsif ($whois =~ /To single out one record, look it up with \"xxx\",/s) {
+        elsif ( $whois =~ /To single out one record, look it up with \"xxx\",/s ) {
             return recursive_whois( "=$dom", $srv, $was_srv );
         }
-        elsif (my ($rs) = _referral_server()) {
+        elsif ( !$norecurse && ( my ( $rs ) = _referral_server() ) ) {
             $newsrv = $rs;
             last;
         }
-        elsif (/Contact information can be found in the (\S+)\s+database/) {
+        elsif ( /Contact information can be found in the (\S+)\s+database/ ) {
             $newsrv = $Net::Whois::Raw::Data::ip_whois_servers{ $1 };
-            }
-        elsif ((/OrgID:\s+(\w+)/ || /descr:\s+(\w+)/) && Net::Whois::Raw::Common::is_ipaddr($dom)) {
+        }
+        elsif ( ( /OrgID:\s+(\w+)/ || /descr:\s+(\w+)/ ) && Net::Whois::Raw::Common::is_ipaddr( $dom ) ) {
             my $val = $1;
-            if($val =~ /^(?:RIPE|APNIC|KRNIC|LACNIC)$/) {
+            if ( $val =~ /^(?:RIPE|APNIC|KRNIC|LACNIC)$/ ) {
                 $newsrv = $Net::Whois::Raw::Data::ip_whois_servers{ $val };
                 last;
             }
-            }
-        elsif (/^\s+Maintainer:\s+RIPE\b/ && Net::Whois::Raw::Common::is_ipaddr($dom)) {
+        }
+        elsif ( /^\s+Maintainer:\s+RIPE\b/ && Net::Whois::Raw::Common::is_ipaddr( $dom ) ) {
             $newsrv = $Net::Whois::Raw::Data::servers{RIPE};
         }
         elsif ( $is_ns && $srv ne $Net::Whois::Raw::Data::servers{NS} ) {
@@ -203,20 +206,26 @@ sub recursive_whois {
         }
     }
 
-    if ($dom =~ /^xn--/i && $newsrv && $Net::Whois::Raw::Data::whois_servers_with_no_idn_support{$newsrv}) {
-        # Bypass recursing to WHOIS servers with no IDN support
+    if (
+        defined $newsrv && (
+            # Bypass recursing to custom servers
+            $Net::Whois::Raw::Data::whois_servers_no_recurse{ $newsrv }
+            # Bypass recursing to WHOIS servers with no IDN support
+            || $dom =~ /^xn--/i && $newsrv && $Net::Whois::Raw::Data::whois_servers_with_no_idn_support{ $newsrv }
+        )
+    ) {
         $newsrv = undef;
     }
 
     my @whois_recs = ( { text => $whois, srv => $srv } );
-    if ($newsrv && $newsrv ne $srv) {
+    if ( $newsrv && $newsrv ne $srv ) {
         warn "recurse to $newsrv\n" if $DEBUG;
 
-        return () if grep {$_ eq $newsrv} @$was_srv;
+        return () if grep { $_ eq $newsrv } @$was_srv;
 
-        my @new_whois_recs = eval { recursive_whois( $dom, $newsrv, [@$was_srv, $srv], 0, $is_ns) };
-        my $new_whois = scalar(@new_whois_recs) ? $new_whois_recs[0]->{text} : '';
-        my $notfound = $Net::Whois::Raw::Data::notfound{$newsrv};
+        my @new_whois_recs = eval { recursive_whois( $dom, $newsrv, [ @$was_srv, $srv ], 0, $is_ns ) };
+        my $new_whois = scalar @new_whois_recs ? $new_whois_recs[0]->{text} : '';
+        my $notfound = $Net::Whois::Raw::Data::notfound{ $newsrv };
 
         if ( $new_whois && !$@ && not ( $notfound && $new_whois =~ /$notfound/im ) ) {
             if ( $is_ns ) {
@@ -227,7 +236,7 @@ sub recursive_whois {
             }
         }
         else {
-                warn "recursive query failed\n" if $DEBUG;
+            warn "recursive query failed\n" if $DEBUG;
         }
     }
 
@@ -250,7 +259,6 @@ sub whois_query {
 
     ### get server for query
     my $server4query = Net::Whois::Raw::Common::get_server($dom);
-    $server4query = lc $server4query;
 
     if ( Net::Whois::Raw::Common::is_ip6addr( $srv ) ) {
         $srv = "[$srv]";
@@ -472,7 +480,7 @@ Net::Whois::Raw - Get Whois information of domains and IP addresses.
 
 =head1 VERSION
 
-version 2.93
+version 2.99037
 
 =head1 SYNOPSIS
 
@@ -557,7 +565,7 @@ Net::Whois::Raw — Get Whois information of domains and IP addresses.
 
 =item whois( DOMAIN [, SRV [, WHICH_WHOIS]] )
 
-Returns Whois information for C<DOMAIN>.
+Returns Whois information for C<DOMAIN> in punycode.
 Without C<SRV> argument default Whois server for specified domain name
 zone will be used. Use 'www_whois' as server name to force
 WHOIS querying via HTTP (only few TLDs are supported in HTTP queries).
@@ -576,7 +584,7 @@ C<WHICH_WHOIS> argument is used to access a results if recursive queries;
 possible values:
 
 C<'QRY_FIRST'> -
-    returns results of the first query. Non't make recursive queries.
+    returns results of the first query. Does not make recursive queries.
     In scalar context returns just whois text.
     In list context returns two values: whois text and whois server
     which was used to make query).
@@ -710,7 +718,7 @@ Alexander Nalobin <alexander@nalobin.ru>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2002-2016 by Alexander Nalobin.
+This software is copyright (c) 2002-2022 by Alexander Nalobin.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
